<?php 
// *****************************************************************************
// test to produce "couldn't fetch mysql" error
// *****************************************************************************

$dbhost     = "localhost";
$dbusername = 'tony';
$dbuserpass = 'tony';

set_error_handler('errorHandler');

// ****************************************************************************
function errorHandler ($errno, $errstr, $errfile, $errline, $errcontext)
// If the error condition is E_USER_ERROR or above then abort.
{
    switch ($errno) {
        case E_USER_WARNING:
        case E_USER_NOTICE:
        case E_WARNING:
            if (isset($errcontext['this'])) {
                if (is_object($errcontext['this'])) {
                    // context is an object
                    if (method_exists($errcontext['this'], 'setErrorString')) {
                        // save error message inside current object
                        $errcontext['this']->setErrorString($errstr);
                    } // if
                } // if
            } // if
        case E_NOTICE:
        case E_CORE_WARNING:
        case E_COMPILE_WARNING:
        case E_STRICT;
            return;
        case E_USER_ERROR:
        case E_ERROR:
        case E_PARSE:
        case E_CORE_ERROR:
        case E_COMPILE_ERROR:
        default:
            break;
    } // switch
    
    if (eregi('^(object)$', $errstr)) {
        if (isset($errcontext['this'])) {
            // retrieve error details from DML object
            if (method_exists($errcontext['this'], 'getQuery')) {
                $query  = $errcontext['this']->getQuery();
            } else {
                $query  = '';
            } // if
            if (method_exists($errcontext['this'], 'getErrorString')) {
                $errstr = $errcontext['this']->getErrorString();
            } else {
                $errstr = '';
            } // if
        } else {
            $query  = null;
        } // if
    } else {
        $query = null;
    } // if
    
    echo "<h2>This application has encountered an unrecoverable error</h2>\n";
    echo "<p>The following has been reported to the administrator:</p>\n";
    echo "<b><font color='red'>";
    echo "<h2>" . date('Y-m-d H:i:s') . "</h2>\n";
    echo "<p>Fatal Error: $errstr (# $errno).</p>\n";
    if ($query) echo "<p>SQL query: $query</p>\n";
    echo "<p>Error in line $errline of file '$errfile'.</p>\n";
    echo "<p>Script: '{$_SERVER['PHP_SELF']}'.</p>\n";
    echo "</b></font>";
    
    // Stop the system
    die();
    
} // errorHandler

// ****************************************************************************

class mysql
// this version is for MySQL (using the 'improved' functions)
{
    var $dbconnect;             // database connection resource
    var $errors = array();      // array of errors
    var $error_string;			// error string
    var $fieldspec;             // field specifications (see class constructor)
    var $lastpage;              // last available page number in current query
    var $numrows;               // number of rows retrieved
    var $pageno;                // requested page number
    var $primary_key;           // array of primary key names
    var $rows_per_page;         // page size for multi-row forms
    var $sql_select;
    var $sql_count;
    var $sql_from;
    var $sql_where;             // additional selection criteria
    var $query;                 // completed DML statement
    
    // ****************************************************************************
    function mysql ($dbname=null)
    {
        if (!empty($dbname)) {
            $result = $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        } else {
            $result = TRUE;
        } // if
        
        return $result;
        
    } // mysql
    
    // ****************************************************************************
    function connect ($dbname)
    // establish a connection to the database
    {
        global $dbhost, $dbusername, $dbuserpass;

        if (!isset($this->dbconnect)) {
            $this->dbconnect = mysqli_connect($dbhost, $dbusername, $dbuserpass);
            if ($this->dbconnect) {
                // use UTF8 for this connection
                $this->query = "SET names 'UTF8'";
                $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
            } // if
        } // if
        if (!isset($this->dbconnect)) {
            return false;
        } elseif (!mysqli_select_db($this->dbconnect, $dbname)) {
            return false;
        } else {
            return true;
        } // if
        
    } // connect
    
    // ****************************************************************************
    function getCount ($dbname, $tablename, $where)
    // get count of records that satisfy selection criteria in $where.
    {
        $this->errors = array();
        
        $this->query = $where;
        
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        $query_data = mysqli_fetch_row($result);
        
        return $query_data[0];
        
    } // getCount
    
    // ****************************************************************************
    function getData ($dbname, $tablename, $where)
    // get data from a database table using optional 'where' criteria.
    // Results may be affected by $where and $pageno.
    {
        $this->errors = array();
        
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        $pageno         = $this->pageno;
        $rows_per_page  = $this->rows_per_page;
        $this->numrows  = 0;
        $this->lastpage = 0;
        
        $array = array();
        
        $select_str = '*';
        $from_str = $tablename;
        $where_str = "WHERE $where";
        
        // count the rows that satisfy this query
        $this->query = "SELECT count(*) FROM $from_str $where_str";
        $this->numrows = $this->getCount($dbname, $tablename, $this->query);
        
        if ($this->numrows <= 0) {
            $this->pageno = 0;
            return $array;
        } // if
        
        if ($rows_per_page > 0) {
            $this->lastpage = ceil($this->numrows / $rows_per_page);
        } else {
            $this->lastpage = 1;
        } // if
        
        if ($pageno == '' OR $pageno <= '1') {
            $pageno = 1;
        } elseif ($pageno > $this->lastpage) {
            $pageno = $this->lastpage;
        } // if
        $this->pageno = $pageno; 
        
        if ($rows_per_page > 0) {
            $limit_str = 'LIMIT ' . ($pageno - 1) * $rows_per_page . ',' . $rows_per_page;
        } else {
            $limit_str = '';
        } // if
        
        $this->query = "SELECT $select_str FROM $from_str $where_str $limit_str";
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        
        while ($row = mysqli_fetch_assoc($result)) {
            $array[] = $row;
        } // while
        
        mysqli_free_result($result);
        
        return $array;
        
    } // std_getData
    
    // ****************************************************************************
    function getErrorString () 
    // return string containing details of last error.
    {
        if (!empty($this->error_string)) {
            $string = $this->error_string;
            $this->error_string = null;
		} elseif ($this->dbconnect) {
            $string = 'MySQL error: ' . mysqli_errno($this->dbconnect) 
                                      . ' - ' 
                                      . mysqli_error($this->dbconnect);
        } else {
            $conerr = mysqli_connect_error();
            if (empty($conerr)) {
                $conerr = getLanguageText('sys0001', $this->dbname); // 'Cannot connect to database'
            } // if
            $string = 'mysqli_connect(): ' . $conerr;
        } // if
        
        return $string;
        
    } // getErrorString
    
    // ****************************************************************************
    function getQuery () 
    // return the last query string that was used
    {
        return $this->query;
        
    } // getQuery
    
    // ****************************************************************************
    function insertRecord ($dbname, $tablename, $fieldarray)
    // insert a record using the contents of $fieldarray.
    {
        $this->errors = array();
        
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        $fieldspec = $this->fieldspec;
        
        // remove any non-database fields from input array
        foreach ($fieldarray as $field => $fieldvalue) {
            // check that $field exists in $fieldspec array
            if (!array_key_exists($field, $fieldspec)) {
                // it does not (like the SUBMIT button, for example), so remove it
                unset ($fieldarray[$field]);
            } // if
        } // foreach
        
        // build 'where' string using values for fields in primary key
        $where = null;
        foreach ($fieldarray as $item => $value) {
            // require this item if it IS part of primary key
            if (in_array($item, $this->primary_key)) {
                $where .= "$item='$value' AND ";
            } // if
        } // foreach
        
        // remove trailing ' AND '
        $where = rtrim($where, ' AND ');
        
        // find out if a record with this primary key already exists
        $this->query = "SELECT count(*) FROM $tablename WHERE $where";
        $count = $this->getCount($dbname, $tablename, $this->query);
        
        // Is this primary key taken?
        if ($count <> 0) {
            // set error message for each field within this key
            foreach ($this->primary_key as $fieldname) {
               $this->errors[$fieldname] = getLanguageText('sys0002'); // 'A record already exists with this ID.'
            } // foreach
            return $fieldarray;
        } // if
        
        // insert this record into the database
        $this->query = "INSERT INTO $tablename SET ";
        foreach ($fieldarray as $item => $value) {
            if (isset($value)) {
                $this->query .= "$item='$value', ";
            } else {
                $this->query .= "$item=NULL, ";
            } // if
        } // foreach
        $this->query = rtrim($this->query, ', ');
        
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        
		return $fieldarray;
        
    } // insertRecord
    
    // ****************************************************************************
    function setErrorString ($string) 
    // this allows a sort order to be specified (see getData)
    {
        $this->error_string = trim($string);
        
    } // setOrderBy
    
// ****************************************************************************
} // end class
// ****************************************************************************

class php_Session
{
    var $dbname;                    // database name
    var $errors = array();          // array of errors
    var $fieldarray;                // array of row/field data
    var $fieldspec = array();       // field specifications (see class constructor)
    var $lastpage;                  // last available page number in current query
    var $message;                   // array of messages
    var $numrows;                   // number of rows retrieved
    var $pageno;                    // requested page number
    var $primary_key = array();     // column(s) which form the primary key
    var $rows_per_page;             // page size for multi-row forms
    var $tablename;                 // table name (internal)
    var $dbms_engine = 'mysql';
    var $dbms_engine_array = array();
    var $where;                     // passed from parent form
    var $sql_select;
    var $sql_from;
    var $sql_where;                 // additional selection criteria
    
    // ****************************************************************************
    function php_Session ()
    {
        // save directory name of current script
        $this->dirname   = dirname(__file__);
        
        $this->tablename = 'php_session';
        $this->dbname    = 'audit';
        
        $this->fieldspec = $this->getFieldSpec_original();
        
    } // php_Session
    
    // ****************************************************************************
    function getFieldSpec_original () 
    // set and return the original specifications for this database table
    // (before being possibly amended at runtime)
    {
        // field specifications for table audit.php_session
        $fieldspec['session_id']                = array('type' => 'string',
                                                        'size' => 32,
                                                        'pkey' => 'y',
                                                        'required' => 'y');
        
        $fieldspec['user_id']                   = array('type' => 'string',
                                                        'size' => 16,
                                                        'uppercase' => 'y');
        
        $fieldspec['date_created']              = array('type' => 'datetime',
                                                        'size' => 20,
                                                        'required' => 'y',
                                                        'default' => '0000-00-00 00:00:00');
        
        $fieldspec['last_updated']              = array('type' => 'datetime',
                                                        'size' => 20,
                                                        'required' => 'y',
                                                        'default' => '0000-00-00 00:00:00');
        
        $fieldspec['session_data']              = array('type' => 'string',
                                                        'size' => 2147483647);
        
        $this->primary_key                      = array('session_id');
        
        return $fieldspec;
        
    } // getFieldSpec_original
    
    // ****************************************************************************
    function getPkeyNames () 
    // return the list of primary key fields in this table.
    {
        if (!empty($this->primary_key)) {
            $array = $this->primary_key;
        } else {
            // construct array from contents of $this->fieldspec
            $array = array();
            foreach ($this->fieldspec as $field => $spec) {
                // look for keyword 'pkey' in field specifications
                if (isset($spec['pkey'])) {
                    $array[] = $field;
                } // if
            } // foreach
        } // if
        
        return $array;
        
    } // getPkeyNames
    
    // ****************************************************************************
    function open ($save_path, $session_name)
    // open the session.
    {
        // do nothing
        return true;
        
    } // open
    
    // ****************************************************************************
    function close ()
    // close the session.
    {
        if (!empty($this->fieldarray)) {
            // perform garbage collection
            $result = $this->gc(0);
            return $result;
        } // if
        
        return FALSE;
        
    } // close
    
    // ****************************************************************************
    function read ($session_id)
    // read any data for this session.
    {
        $fieldarray = $this->_dml_getData("session_id='$session_id'");
        
        if (isset($fieldarray[0]['session_data'])) {
            $this->fieldarray = $fieldarray[0];
            $this->fieldarray['session_data'] = '';
            return $fieldarray[0]['session_data'];
        } else {
            return '';  // return an empty string
        } // if
        
    } // read
    
    // ****************************************************************************
    function write ($session_id, $session_data)
    // write session data to the database.
    {
        if (!empty($this->fieldarray)) {
            if ($this->fieldarray['session_id'] != $session_id) {
                // user is starting a new session with previous data
                $this->fieldarray = array();
            } // if
        } // if
            
        if (empty($this->fieldarray)) {
            // create new record
            $array['session_id']   = $session_id;
            $array['date_created'] = date('Y-m-d H:i:s');
            $array['last_updated'] = date('Y-m-d H:i:s');
            $array['session_data'] = addslashes($session_data);
            $this->_dml_insertRecord($array);
        } else {
            // update existing record
            if (isset($_SESSION['logon_user_id'])) {
                $array['user_id']      = $_SESSION['logon_user_id'];
            } // if
            $array['last_updated'] = date('Y-m-d H:i:s');
            $array['session_data'] = addslashes($session_data);
            $this->_dml_updateRecord($array, $this->fieldarray);
        } // if
        
        return true;
        
    } // write
    
    // ****************************************************************************
    function destroy ($session_id)
    // destroy the specified session.
    {
        $fieldarray['session_id'] = $session_id;
        $this->_dml_deleteRecord($fieldarray);
        
        return true;
        
    } // destroy
    
    // ****************************************************************************
    function gc ($max_lifetime)
    // perform garbage collection.
    {
        $real_now = date('Y-m-d H:i:s');
        $dt1 = strtotime("$real_now -2 hours");
        $dt2 = date('YmdHis', $dt1);
        
        $count = $this->_dml_deleteSelection("last_updated < $dt2");
        
        return true;
        
    } // gc
    
    // ****************************************************************************
    function _dml_getData ($where)
    // Get data from the specified database table.
    // Results may be affected by $where and $pageno.
    {
        $DML = $this->_getDBMSengine();
        
        $DML->fieldspec     = $this->fieldspec;
        $DML->pageno        = $this->pageno;
        $DML->rows_per_page = $this->rows_per_page;
        $DML->sql_from      = $this->sql_from;
        $DML->sql_select    = $this->sql_select;
        $DML->sql_where     = $this->sql_where;
        
        $array = $DML->getData($this->dbname, $this->tablename, $where);
        
        return $array;
        
    } // _dml_getData
    
    // ****************************************************************************
    function _dml_insertRecord ($fieldarray)
    // insert a record using the contents of $fieldarray.
    {
        $DML = $this->_getDBMSengine();
        
        $DML->fieldspec = $this->getFieldSpec_original();
        
        $DML->primary_key = $this->getPkeyNames();
        
        $array = $DML->insertRecord($this->dbname, $this->tablename, $fieldarray);
        
        return $array;
        
    } // _dml_insertRecord
    
    // ****************************************************************************
    function _getDBMSengine ()
    // get/create the object that deals with this database engine.
    {
        foreach ($this->dbms_engine_array as $name => $object) {
            if ($name == $this->dbms_engine) {
                $DML = $object;
                return $DML;
            } // if
        } // foreach
        
        $this->dbms_engine_array[$this->dbms_engine] = new mysql($this->dbname);
        
        $DML = $this->dbms_engine_array[$this->dbms_engine];
        return $DML;
        
    } // _getDBMSengine
    
// ****************************************************************************
} // end class
// ****************************************************************************

$session_class = new php_Session;
session_set_save_handler(array(&$session_class, 'open'),
                         array(&$session_class, 'close'),
                         array(&$session_class, 'read'),
                         array(&$session_class, 'write'),
                         array(&$session_class, 'destroy'),
                         array(&$session_class, 'gc'));

session_start();

$title = "test to produce 'couldnt fetch mysqli' error";
$help_text = "This is a piece of text";

echo "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.0 Transitional//EN' 'http://www.w3.org/TR/html4/loose.dtd'>\n";
echo "<html>\n";
echo "<head>\n";
echo "  <title>$title</title>\n";
echo "</head>\n";
echo "<body>\n";
echo "<div class='body'>\n";
echo "<h1>$title</h1>\n";
echo $help_text . "\n";
echo "</div>\n";
echo "</body>\n";
echo "</html>\n";

exit;

?>
